@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 #
-# $Id: test_happydoc.py,v 1.74 2001/12/16 12:13:53 doughellmann Exp $
+# $Id: test_happydoc.py,v 1.88 2002/08/24 17:54:02 doughellmann Exp $
 #
 # Copyright Doug Hellmann 2000
 #
@@ -33,7 +33,7 @@ __rcs_info__ = {
     #  Creation Information
     #
     'module_name'  : '$RCSfile: test_happydoc.py,v $',
-    'rcs_id'       : '$Id: test_happydoc.py,v 1.74 2001/12/16 12:13:53 doughellmann Exp $',
+    'rcs_id'       : '$Id: test_happydoc.py,v 1.88 2002/08/24 17:54:02 doughellmann Exp $',
     'creator'      : 'Doug Hellmann <doug@hellfly.net>',
     'project'      : 'HappyDoc',
     'created'      : 'Sun, 13-Aug-2000 10:16:13 EDT',
@@ -42,8 +42,8 @@ __rcs_info__ = {
     #  Current Information
     #
     'author'       : '$Author: doughellmann $',
-    'version'      : '$Revision: 1.74 $',
-    'date'         : '$Date: 2001/12/16 12:13:53 $',
+    'version'      : '$Revision: 1.88 $',
+    'date'         : '$Date: 2002/08/24 17:54:02 $',
 }
 try:
     __version__ = __rcs_info__['version'].split(' ')[1]
@@ -55,6 +55,7 @@ except:
 #
 import sys
 import os
+sys.path.append(os.getcwd())
 import tempfile
 import string
 from glob import glob
@@ -69,6 +70,15 @@ from happydoclib.appclass import HappyDoc
 from happydoclib.StreamFlushTest import StreamFlushTest, DEFAULT_OUTPUT_DIR
 from happydoclib.StreamFlushTest import verboseLevel as globalVerboseLevel
 
+WEB_CHECKER_DIR='../Python-2.2/Tools/webchecker/'
+
+if os.path.exists(WEB_CHECKER_DIR) and (os.name == 'posix'):
+    sys.path.append(WEB_CHECKER_DIR)
+    import webchecker
+    WEBCHECKER_AVAILABLE=1
+else:
+    WEBCHECKER_AVAILABLE=0
+
 #
 # Modules with tests
 #
@@ -87,23 +97,29 @@ import happydoclib.trace
 #
 # Module
 #
-
     
     
 class HappyDocTestBase(StreamFlushTest):
+    
+    WEB_CHECKER = os.path.join(WEB_CHECKER_DIR, 'webchecker.py')
 
-    def setUp(self):
-        self.happydoc = './happydoc'
+    def __init__(self, methodName='runTest', outputDir=DEFAULT_OUTPUT_DIR,
+                 statusMessageFunc=None):
+        StreamFlushTest.__init__(self, methodName, outputDir, statusMessageFunc)
         return
 
-    def runHappyDoc(self, modules=(), extraArgs=()):
+    def setUp(self):
+        self.happydoc = os.path.join(os.curdir, 'happydoc')
+        return
+
+    def runHappyDoc(self, modules=(), extraArgs=(), useWebChecker=0):
         #
         # Fix up output directory variable
         #
         output_dir = self.output_dir
         happydoc = self.happydoc
         test_name = self.name
-        output_dir = '%(output_dir)s/%(test_name)s' % locals()
+        output_dir = os.path.join(output_dir, test_name)
         #
         # Verbose level setting
         #
@@ -152,6 +168,41 @@ class HappyDocTestBase(StreamFlushTest):
         except HappyDoc.HelpRequested, msg:
             if msg:
                 exit_code = 1
+
+        #
+        # Run webchecker to verify the links that were written to the
+        # output.
+        #
+        if useWebChecker and WEBCHECKER_AVAILABLE:
+
+            if self.verboseLevel.get():
+                print
+                print 'WebChecker:'
+                print
+            c = webchecker.Checker()
+            c.setflags( checkext=0, verbose=2, nonames=1 )
+
+            #
+            # Work out the output prefix, if one was specified.  This is
+            # used to determine the real root filename, since it will
+            # also include the prefix value.
+            #
+            prefix = ''
+            for extra_arg in extraArgs:
+                try:
+                    name, value = extra_arg.split('=')
+                except ValueError:
+                    pass
+                else:
+                    if name == 'formatter_filenamePrefix':
+                        prefix = value
+                        break
+            
+            url = os.path.join(output_dir, '%sindex.html' % prefix)
+            c.addroot( url )
+            c.run()
+            c.report()
+            assert not c.bad, "Link checker reports errors."
         
         #
         # Flush the output so watchers can keep up
@@ -194,68 +245,117 @@ class ExecuteHappyDocTest(HappyDocTestBase):
     # Formatter and docset tests
     #
     
+    
+    def testDocBookX(self):
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('-F',
+                                                 'docbookx')
+                                      )
+                ), 'Full self documentation in DocBookS format output test failed.'
+        return
+    
+    def testHTMLSimple(self):
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('-p', '-',),
+                                      useWebChecker=1,
+                                      )
+                ), 'Basic single-file docset test failed.'
+        return
+    
+    def testHTMLSimpleWithPackageDescriptionFiles(self):
+        assert (not self.runHappyDoc( (os.path.join('TestCases',
+                                                    'test_package_summaries'),
+                                       ),
+                                      useWebChecker=1,
+                                      )
+                ), 'Basic docset test with package description files failed.'
+        return
+    
+    def testSelfHTMLCompact(self):
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('formatter_compactHTML=yes',),
+                                      useWebChecker=1,
+                                      )
+                ), 'Full self documentation with compact output test failed.'
+        return
+    
     def testHTMLSingleFile(self):
-        assert (not self.runHappyDoc( ('TestCases/test.py',),
-                                      extraArgs=('-p', '-', '-T', 'SingleFile')
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('-p', '-', '-T', 'SingleFile'),
+                                      useWebChecker=1,
                                       )
                 ), 'Basic single-file docset test failed.'
         return
     
     def testHTMLSingleFileCompact(self):
-        assert (not self.runHappyDoc( ('TestCases/test.py',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
                                       extraArgs=('-p', '-', '-T', 'SingleFile',
                                                  'formatter_compactHTML=yes',
-                                                 )
+                                                 ),
+                                      useWebChecker=1,
                                       )
                 ), 'Basic single-file docset test failed.'
         return
 
     def testHTMLStdout(self):
-        assert (not self.runHappyDoc( ('TestCases/test.py',),
-                                      extraArgs=('-T', 'StdOut')
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('-T', 'StdOut'),
+                                      )
+                ), 'HTML to standard-output docset test failed.'
+        return
+
+    def testDashOStdOut(self):
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
+                                      extraArgs=('-o',)
                                       )
                 ), 'HTML to standard-output docset test failed.'
         return
 
     def testText(self):
-        assert (not self.runHappyDoc( ('TestCases/test.py',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
                                       extraArgs=('-F', 'Text')
                                       )
                 ), 'Text formatter test failed.'
         return
 
     def testTextStdout(self):
-        assert (not self.runHappyDoc( ('TestCases/test.py',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test.py'),),
                                       extraArgs=('-F', 'Text', '-T', 'StdOut')
                                       )
                 ), 'Text to standard-output docset test failed.'
         return
 
     def testDia(self):
-        assert (not self.runHappyDoc( ('TestCases/test_dia.py', ),
+        assert (not self.runHappyDoc( (os.path.join('TestCases', 'test_dia.py'), ),
                                       extraArgs=('--dia',)
                                       )
                 ), 'Dia output test failed.'
         return
 
     def testImportsFromPackages(self):
-        assert (not self.runHappyDoc( ('TestCases/test_import_packages_basic',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases',
+                                                    'test_import_packages_basic'),),
+                                      useWebChecker=1,
                                       )
                 ), 'Import from packages test failed.'
         return
 
     def testBasicImportsFromPackagesIgnorePackages(self):
-        assert (not self.runHappyDoc( ('TestCases/test_import_packages',),
-                                      extraArgs=('docset_usePackages=0',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases',
+                                                    'test_import_packages'),),
+                                      extraArgs=('docset_usePackages=0',), 
+                                      useWebChecker=1,
                                       )
                 ), 'Import from packages while ignoring package special handling test failed.'
         return
 
     def testOutputWithPrefix(self):
-        assert (not self.runHappyDoc( ('TestCases/test_import_packages_basic',),
+        assert (not self.runHappyDoc( (os.path.join('TestCases',
+                                                    'test_import_packages_basic'),),
                                       extraArgs=('-p',
                                                  '',
-                                                 'formatter_filenamePrefix=TESTPREFIX_')
+                                                 'formatter_filenamePrefix=TESTPREFIX_'),
+                                      useWebChecker=1,
                                       )
                 ), 'Formatter output prefix test failed.'
         return 
@@ -264,53 +364,39 @@ class ExecuteHappyDocTest(HappyDocTestBase):
     ## Full self documentation tests
     ##
     
-#     def testSelfDia(self):
-#         assert (not self.runHappyDoc( ('../HappyDoc',),
-#                                       extraArgs=('--dia',))
-#                 ), 'Full self documentation test in dia format failed.'
-#         return
-   
     def testSelfHTML(self):
-        assert (not self.runHappyDoc( ('../HappyDoc',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'HappyDoc'),),
+                                      useWebChecker=1,
                                       )
                 ), 'Full self documentation test failed.'
         return
     
-#     def testSelfHTMLCompact(self):
-#         assert (not self.runHappyDoc( ('../HappyDoc',),
-#                                       extraArgs=('formatter_compactHTML=yes',))
-#                 ), 'Full self documentation with compact output test failed.'
-#         return
-    
-    def testSelfDocBookSingleFile(self):
-        assert (not self.runHappyDoc( ('../HappyDoc',),
+    def testSelfSGMLDocBookSingleFile(self):
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'HappyDoc'),),
                                       extraArgs=('-F',
                                                  'SGMLDocBook',
                                                  '-T',
-                                                 'SingleFile'))
-                ), 'Full self documentation in DocBook format output test failed.'
+                                                 'SingleFile'),
+                                      )
+                ), 'Full self documentation in SGMLDocBook format output test failed.'
         return
-
-#     def testSelfDocBookXML(self):
-#         assert (not self.runHappyDoc( ('../HappyDoc',),
-#                                       extraArgs=('-F',
-#                                                  'docbookx',
-#                                                  '-T',
-#                                                  'mstruct'))
-#                 ), 'Full self documentation in DocBookX format output test failed.'
-#         return 
-    
-#     def testSelfPlainText(self):
-#         assert (not self.runHappyDoc( ('../HappyDoc',),
-#                                       extraArgs=('-F', 'Text'))
-#                 ), 'Full self documentation-as-text test failed.'
-#         return
     
+    def testSelfDocBookXSingleFile(self):
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'HappyDoc'),),
+                                      extraArgs=('-F',
+                                                 'docbookx',
+                                                 '-T',
+                                                 'SingleFile'),
+                                      )
+                ), 'Full self documentation in DocBookS format output test failed.'
+        return
+        
 
 class OtherWorkingDirTest(HappyDocTestBase):
 
     def __init__(self,
-                 workingDir='.',
+                 methodName='runTest',
+                 workingDir=os.curdir,
                  outputDir='DefaultTestOutputDir',
                  **nargs
                  ):
@@ -319,6 +405,7 @@ class OtherWorkingDirTest(HappyDocTestBase):
         #
         output_dir = happydoclib.path.join(os.pardir, 'HappyDoc', outputDir)
         nargs['outputDir'] = output_dir
+        nargs['methodName'] = methodName
         apply(HappyDocTestBase.__init__, (self,), nargs)
         #
         # This class
@@ -328,7 +415,7 @@ class OtherWorkingDirTest(HappyDocTestBase):
         return
 
     def setUp(self):
-        self.happydoc = '../HappyDoc/happydoc'
+        self.happydoc = os.path.join(os.pardir, 'HappyDoc', 'happydoc')
         return
     
     def runHappyDoc(self, *args, **nargs):
@@ -357,7 +444,9 @@ class ExternalTest(HappyDocTestBase):
         assert not ret, 'External test command "%s" failed' % command
 
     def testPluginLoader(self):
-        self.externalApp('./TestCases/test_plugin_loader/runtest.py')
+        self.externalApp(os.path.join(os.curdir, 'TestCases',
+                                      'test_plugin_loader',
+                                      'runtest.py'))
         return
 
 
@@ -365,31 +454,45 @@ class ExternalTest(HappyDocTestBase):
 class ZopeTest(OtherWorkingDirTest):
     
     def testZopeFull(self):
-        assert (not self.runHappyDoc( ('../Zope-2-CVS-src',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'Zope-2-CVS-src'),),
                                       )
                 ), 'Zope full documentation test failed.'
         return
     
     def testZopeRoot(self):
-        assert (not self.runHappyDoc( ('../Zope-2-CVS-src',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'Zope-2-CVS-src'),),
                                       extraArgs=('-r',))
                 ), 'Zope full documentation test failed.'
         return
 
     def testGadflyParseError(self):
-        assert (not self.runHappyDoc( ('../Zope-2-CVS-src/lib/python/Products/ZGadflyDA/gadfly/gfdb0.py',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir,
+                                                    'Zope-2-CVS-src',
+                                                    'lib',
+                                                    'python',
+                                                    'Products',
+                                                    'ZGadflyDA',
+                                                    'gadfly',
+                                                    'gfdb0.py',
+                                                    ),),
                                       extraArgs=('-r',))
                 ), 'Gadfly test with parse-error failed.'
         return
 
     def testZEOParseError(self):
-        assert (not self.runHappyDoc( ('../Zope-2-CVS-src/lib/python/ZEO/zrpc.py',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir,
+                                                    'Zope-2-CVS-src',
+                                                    'lib',
+                                                    'python',
+                                                    'ZEO',
+                                                    'zrpc.py',
+                                                    ),),
                                       extraArgs=('-r',))
                 ), 'ZEO test with parse-error failed.'
         return
 
     def testZopeWithSafePrefix(self):
-        assert (not self.runHappyDoc( ('../Zope-2-CVS-src',),
+        assert (not self.runHappyDoc( (os.path.join(os.pardir, 'Zope-2-CVS-src'),),
                                       extraArgs=('formatter_filenamePrefix=zsd_',))
                 ), 'Zope test with output prefix failed.'
         return
@@ -397,7 +500,8 @@ class ZopeTest(OtherWorkingDirTest):
         
         
 def ZopeTestFactory(**nargs):
-    nargs['workingDir'] = nargs.get('workingDir', '../Zope-2-CVS-src')
+    nargs['workingDir'] = nargs.get('workingDir', os.path.join(os.pardir,
+                                                               'Zope-2-CVS-src'))
     return apply(ZopeTest, (), nargs)
 
 
@@ -415,7 +519,8 @@ class HappyDocBugRegressionTest(HappyDocTestBase):
 
     def checkBugReport(self, bugId):
         print '\n%s: %s' % (bugId, os.getcwd())
-        assert not self.runHappyDoc( ('./TestCases/test_bug%s.py' % bugId, ),
+        assert not self.runHappyDoc( (os.path.join(os.curdir, 'TestCases',
+                                                   'test_bug%s.py' % bugId), ),
                                      extraArgs=('-p', '-'),
                                      ), 'Check for bug %s failed.' % bugId
 
@@ -438,8 +543,10 @@ class HappyDocTestLoader(unittest.TestLoader):
     between standard test output and regression test output.
     """
 
-    def __init__(self, outputDir):
+    def __init__(self, outputDir='', statusMessageFunc=None, knownTestCaseClasses=[]):
         self.output_dir = outputDir
+        self.status_message_func = statusMessageFunc
+        self.known_classes = knownTestCaseClasses
         return
 
     def loadTestsFromTestCase(self, testCaseClass):
@@ -447,9 +554,129 @@ class HappyDocTestLoader(unittest.TestLoader):
         names = self.getTestCaseNames(testCaseClass)
         tests = []
         for n in names:
-            tests.append( testCaseClass(n, self.output_dir) )
+            try:
+                test_case = testCaseClass(n,
+                                          outputDir=self.output_dir,
+                                          statusMessageFunc=self.status_message_func,
+                                          )
+            except TypeError:
+                test_case = testCaseClass(n, outputDir=self.output_dir)
+                
+            tests.append(test_case)
         return self.suiteClass(tests)
+
+    def loadTestsFromName(self, name, module=None):
+        try:
+            test_obj = unittest.TestLoader.loadTestsFromName(self, name, module)
+            test_obj.status_message_func = self.status_message_func
+            return test_obj
+        except ImportError:
+            for checking_class in self.known_classes:
+                if name in self.getTestCaseNames(checking_class):
+                    return self.suiteClass([
+                        checking_class(name,
+                                       outputDir=self.output_dir,
+                                       statusMessageFunc=self.status_message_func)
+                        ])
+                
+
+def addAllTests(test_loader, actual_test_suite):
+    #
+    # Load tests from modules we know contain tests
+    #
+    for m in (
+        #
+        # Supporting tools
+        #
+        happydoclib.CommandLineApp,
+        happydoclib.indentstring,
+        happydoclib.optiontools,
+        happydoclib.trace,
+        happydoclib.path,
+
+        #
+        # Foundation modules
+        #
+        happydoclib.parsecomments,
+        happydoclib.parseinfo,
+        happydoclib.happydocstring,
+        happydoclib.happydocset,
+
+        #
+        # Docstring converters
+        #
+        happydoclib.docstring,
+        happydoclib.docstring.docstring_ClassicStructuredText,
+        happydoclib.docstring.docstring_StructuredText,
+        happydoclib.docstring.docstring_RawText,
+        happydoclib.docstring.docstring_PlainText,
+
+        #
+        # Formatters
+        #
+        happydoclib.formatter.fileformatterbase,
+        happydoclib.formatter.openoffice,
+        happydoclib.formatter.formatter_HTMLFile,
+
+        #
+        # Docsets
+        #
+        happydoclib.docset.docset_MultipleFile
+
+        ):
+        actual_test_suite.addTest(test_loader.loadTestsFromModule(m))
+    #
+    # Load tests from classes in this module
+    #
+    for c in ( ExecuteHappyDocTest,
+               ExternalTest,
+               ):
+        actual_test_suite.addTest(test_loader.loadTestsFromTestCase(c))
+    return
+
+
+
+def getBugTests(actual_test_suite, output_dir):
+    bug_ids = map(lambda x:x[18:-3], glob(os.path.join('TestCases', 'test_bug*.py')))
+    bug_ids.sort()
+    for bug in bug_ids:
+        #test_definitions.append( (bug, HappyDocBugRegressionTest) )
+        actual_test_suite.addTest(
+            HappyDocBugRegressionTest( methodName=bug,
+                                       outputDir=output_dir,
+                                       )
+            )
+    return
+
+def getTestSuite(get_all_tests=1, get_bug_tests=1, output_dir='/dev/null',
+                 progress_callback=None, args=[]):
+    actual_test_suite = unittest.TestSuite()
+
+    test_loader = HappyDocTestLoader(output_dir,
+                                     progress_callback,
+                                     (ExecuteHappyDocTest,
+                                      ZopeTest,
+                                      HappyDocBugRegressionTest,
+                                      ))
+
+    if get_all_tests:
+        addAllTests(test_loader, actual_test_suite)
+        getBugTests(actual_test_suite, output_dir)
+
+    elif get_bug_tests:
+        getBugTests(actual_test_suite, output_dir)
         
+    elif args:
+        # Run the tests specified by the user
+        for a in args:
+            actual_test_suite.addTest( test_loader.loadTestsFromName(a) )
+    else:
+        # Default to running all tests
+        get_all_tests = 0
+        actual_test_suite.addTest( test_loader.loadTestsFromName('__main__.ExecuteHappyDocTest.testSelfHTML') )
+
+    return actual_test_suite
+
 
 class TestCaseDriver(happydoclib.CommandLineApp.CommandLineApp):
     "Drive the test cases for HappyDoc."
@@ -471,6 +698,15 @@ class TestCaseDriver(happydoclib.CommandLineApp.CommandLineApp):
     ## OPTION HANDLERS
     ##
 
+    log_file=open('log.txt', 'wt')
+
+    def statusMessage(self, msg='', level=1):
+        happydoclib.CommandLineApp.CommandLineApp.statusMessage(
+            self, msg, level)
+        self.log_file.write('%s\n' % msg)
+        self.log_file.flush()
+        return
+
     def optionHandler_d(self, outputDir):
         "Specify the output directory for the tests."
         self.statusMessage('Setting output directory to "%s"' % outputDir, 2)
@@ -562,91 +798,25 @@ class TestCaseDriver(happydoclib.CommandLineApp.CommandLineApp):
         #
         # Figure out what tests to process
         #
-        actual_test_suite = unittest.TestSuite()
-
-        test_loader = HappyDocTestLoader(self._output_dir)
-
         get_all_tests = 0
+        get_bug_tests = 0
         if 'all' in args:
             # We will define "all" of the tests later
             get_all_tests = 1
-        elif args:
-            # Run the tests specified by the user
-            for a in args:
-                actual_test_suite.addTest( test_loader.loadTestsFromName(a) )
-        else:
-            # Default to running all tests
-            get_all_tests = 0
-            actual_test_suite.addTest( test_loader.loadTestsFromName('__main__.ExecuteHappyDocTest.testSelfHTML') )
-
-        if get_all_tests:
-            #
-            # Load tests from modules we know contain tests
-            #
-            for m in (
-                #
-                # Supporting tools
-                #
-                happydoclib.CommandLineApp,
-                happydoclib.indentstring,
-                happydoclib.optiontools,
-                happydoclib.trace,
-                happydoclib.path,
-
-                #
-                # Foundation modules
-                #
-                happydoclib.parsecomments,
-                happydoclib.parseinfo,
-                happydoclib.happydocstring,
-                happydoclib.happydocset,
-
-                #
-                # Docstring converters
-                #
-                happydoclib.docstring,
-                happydoclib.docstring.docstring_ClassicStructuredText,
-                happydoclib.docstring.docstring_StructuredText,
-                happydoclib.docstring.docstring_RawText,
-                happydoclib.docstring.docstring_PlainText,
-
-                #
-                # Formatters
-                #
-                happydoclib.formatter.openoffice,
-                happydoclib.formatter.formatter_HTMLFile,
-
-                #
-                # Docsets
-                #
-                happydoclib.docset.docset_MultipleFile
-                
-                ):
-                actual_test_suite.addTest(test_loader.loadTestsFromModule(m))
-            #
-            # Load tests from classes in this module
-            #
-            for c in ( ExecuteHappyDocTest,
-                       ExternalTest,
-                       ):
-                actual_test_suite.addTest(test_loader.loadTestsFromTestCase(c))
-            #
-            # Check tests related to bug reports
-            #
-            bug_ids = map(lambda x:x[18:-3], glob('TestCases/test_bug*.py'))
-            bug_ids.sort()
-            for bug in bug_ids:
-                #test_definitions.append( (bug, HappyDocBugRegressionTest) )
-                actual_test_suite.addTest(
-                    HappyDocBugRegressionTest( methodName=bug,
-                                               outputDir=self._output_dir,
-                                               )
-                    )
-            #
-            # Optionally include the Zope tests
-            #
-            if self._include_zope:
-                actual_test_suite.addTest(test_loader.loadTestsFromTestCase(ZopeTest))
+        elif 'bugs' in args:
+            get_bug_tests = 1
+            
+        actual_test_suite = getTestSuite(get_all_tests=get_all_tests,
+                                         get_bug_tests=get_bug_tests,
+                                         output_dir=self._output_dir,
+                                         progress_callback=self.statusMessage,
+                                         args=args)
+        
+        #
+        # Optionally include the Zope tests
+        #
+        if self._include_zope:
+            actual_test_suite.addTest(test_loader.loadTestsFromTestCase(ZopeTest))
                 
         #
         # Figure out what action to take
@@ -659,6 +829,7 @@ class TestCaseDriver(happydoclib.CommandLineApp.CommandLineApp):
             raise ValueError('Operation (%s) must be one of RUNTEST or LIST.' % \
                              self._operation)
         
+        self.log_file.close()
         return
     
 
@@ -670,9 +841,17 @@ def main(argv=()):
 
 
 def debug():
-    main( ('-t', 'checkHelpSyntax') )
+    os.chdir('c:\\happydoc\happydoc')
+    import happydoclib.trace
+    #happydoclib.trace.trace.setVerbosity(3)
+    main( ('all',) )
+    #main( ('-v',
+    #       'happydoclib.formatter.fileformatterbase.FileFormatterBaseTest.testGetFullOutputNameForObjectNone') )
     return
     
 
 if __name__ == '__main__':
-    main(sys.argv[1:])
+    if os.name == 'nt':
+        debug()
+    else:
+        main(sys.argv[1:])
